#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>


#include <vm68k/types.h>
#include <vm68k/memory.h>
#include <vm68k/processor.h>


#include "programram.hh"

using namespace vm68k ;

const uint32_type   memorySize = 0x10000 ;
const char          fileName [] = "a.out" ;

static void bitOutput ( FILE * output , uint32_type data , int numberOfBits )
{
    int    bit ;

    for ( bit = numberOfBits - 1 ; bit >= 0 ; bit -- )
    {
        fputc ( (( data >> bit ) & 1 ) ? '1' : '0' , output ) ;
    }
}
        



int main ( int argc , char * argv [] )
{
    ProgramRAM *        prog ;
    FILE *              output = stdout ;
    int                 numberOfBits ;
    uint32_type         pos ;
    int                 bitWidth = 8 ;
    int                 bitWidthOffset ;

    if ( argc >= 2 )
    {
        bitWidth = atoi ( argv [ 1 ] ) ;
    } 
    switch ( bitWidth )
    {
        case 8 :    bitWidthOffset = 0 ;
                    break ;
        case 16 :   bitWidthOffset = 1 ;
                    break ;
        case 32 :   bitWidthOffset = 2 ;
                    break ;
        default :   fprintf ( stderr , "Invalid bit width specified. "
                            "Must be 8, 16 or 32.\n" ) ;
                    return 1 ; 
    }

    try
    {
        prog = new ProgramRAM ( 0 , memorySize , fileName ) ;
    } catch ( exception e )
    {
        fprintf ( stderr , "Unable to read file '%s'.\n" , fileName ) ;
        return 1 ;
    }

    /* how many bits are actually required to represent this program? */
    {
        int i = prog -> getHighestAddrUsedByProgram () ;

        numberOfBits = 0 ;
        while ( i != 0 )
        {
            i >>= 1 ;
            numberOfBits ++ ;
        }
    }
    fprintf ( stderr , "Program '%s': starts at 0x%0x, ends at 0x%0x, %d bits\n" 
        "Program entry point: 0x%0x\n"
        "Generating VHDL wrapper for data width %d bits..." ,
        fileName , prog -> getLowestAddrUsedByProgram () ,
        prog -> getHighestAddrUsedByProgram () , numberOfBits ,
        prog -> getStartPC () , bitWidth ) ;

    fprintf ( output ,
        "-- autogenerated by romfactory\n\n"
        "library ieee ;\n"
        "use ieee . std_logic_1164 . all ;\n\n"
        "entity program_rom is\n"
        "   port ( address : in std_logic_vector ( 31 downto 0 ) ;\n"
        "          clock : in std_logic ;\n"
        "          data : out std_logic_vector ( %d downto 0 ) ) ;\n"
        "end entity program_rom ;\n\n\n"
        "architecture basic of program_rom is\n"
        "   subtype ad is std_logic_vector ( %d downto %d ) ;\n"
        "   subtype dt is std_logic_vector ( %d downto 0 ) ;\n"
        "begin\n"
        "   program_rom_process : process ( address ,\n"
        "           clock ) is\n"
        "   begin\n"
        "       if ( clock = '1' )\n"
        "       and ( clock'event )\n"
        "       then\n"
        "           case address( %d downto %d ) is\n" , 
            bitWidth - 1 , 
            numberOfBits - 1 , bitWidthOffset , 
            bitWidth - 1 ,
            numberOfBits - 1 , bitWidthOffset ) ;


    for ( pos = prog -> getLowestAddrUsedByProgram () ;
            pos <= prog -> getHighestAddrUsedByProgram () ; 
            pos += ( 1 << bitWidthOffset ) )
    {
        uint32_type data ;
        
        switch ( bitWidth )
        {
            case 8 :    data = prog -> get_8 ( pos , 
                            (vm68k::memory::function_code) 0 ) ;
                        break ;
            case 16 :   data = prog -> get_16 ( pos , 
                            (vm68k::memory::function_code) 0 ) ;
                        break ;
            case 32 :   data = prog -> get_32 ( pos , 
                            (vm68k::memory::function_code) 0 ) ;
                        break ;
            default :   assert ( 0 ) ;
                        break ;
        }

        if ( data != 0 ) 
        {
            /* if the byte is zero, the others clause will catch it.
            This is good. Many of the bytes will be zero. */
            fprintf ( output , "when ad'(\"" ) ;
            bitOutput ( output , pos >> bitWidthOffset , 
                                numberOfBits - bitWidthOffset ) ;
            fprintf ( output , "\") => data <= dt'(\"" ) ;
            bitOutput ( output , data , bitWidth ) ;
            fprintf ( output , "\") ; -- %x\n" , data ) ;
        }
    }
    fprintf ( output , "when others => data <= dt'(\"" ) ;
    bitOutput ( output , 0 , bitWidth ) ;
    fprintf ( output , "\") ;\n" ) ;
    fprintf ( output , "end case ;\n" ) ;

    fprintf ( output ,
            "       end if ;\n"
            "   end process program_rom_process ;\n"
            "end architecture basic ;\n\n\n" ) ;

    fprintf ( stderr , "done\n" ) ;

    return 0 ;
}

