    
    
    
    alu_input_muxes : process ( alu_source_a , alu_source_b , 
                pc_register ,
                register_file_address_out_x , register_file_address_out_y ,
                register_file_data_out_x , register_file_data_out_y ,
                operand_value , operand_address ,
                alu_input_small_number ,
                immediate_data_reg ,
                instruction_register ) is
    begin
        -- MUX for alu_input_a:-
        case alu_source_a is
        when ALU_A_PC =>
                    alu_input_a <= pc_register ;
        when ALU_A_ADDRESS_X =>
                    alu_input_a <= register_file_address_out_x ;
        when ALU_A_DATA_X =>
                    alu_input_a <= register_file_data_out_x ;
        when ALU_A_OPERAND_VALUE =>
                    alu_input_a <= operand_value ;
        when others => -- ALU_A_PGI =>
                    alu_input_a ( 3 downto 0 ) <= alu_input_small_number ;
                    alu_input_a ( 31 downto 4 ) <= ( others => '0' ) ;
        end case ;

        -- MUX for alu_input_b:-
        case alu_source_b is
        when ALU_B_PGI =>
                    -- A small number from 0 to 15. We do it with a separate
                    -- signal, alu_input_small_number, so that a large
                    -- 32 bit mux will not be synthesised just to switch
                    -- between 4 bit numbers.
                    alu_input_b ( 3 downto 0 ) <= alu_input_small_number ;
                    alu_input_b ( 31 downto 4 ) <= ( others => '0' ) ;
        when ALU_B_IDR =>
                    alu_input_b <= immediate_data_reg ;
        when ALU_B_ADDRESS_Y =>
                    alu_input_b <= register_file_address_out_y ;
        when ALU_B_DATA_Y =>
                    alu_input_b <= register_file_data_out_y ;
        when ALU_B_OA =>
                    alu_input_b <= operand_address ;
        when others => -- ALU_B_LOW_BYTE_OF_IR =>
                    -- IR(7..0) sign extended to 32 bits - used for branches
                    alu_input_b ( 7 downto 0 ) <= 
                            instruction_register ( 7 downto 0 ) ;
                    alu_input_b ( 31 downto 8 ) <= 
                            ( others => instruction_register ( 7 ) ) ;
        end case ;
    end process alu_input_muxes ;

    alu_input_muxes_2 : process ( pgi_source , operation_size ,
                    instruction_register , ea_reg ) is
        variable qim        : std_logic_vector ( 4 downto 0 ) ;
        variable postinc    : std_logic_vector ( 4 downto 0 ) ;
    begin
        -- MUX for alu_input_small_number:-
        case pgi_source is
        when PGI_ZERO =>
                    alu_input_small_number <= "0000" ;
        when PGI_ONE =>
                    alu_input_small_number <= "0001" ;
        when PGI_TWO =>
                    alu_input_small_number <= "0010" ;
        when PGI_THREE =>
                    alu_input_small_number <= "0011" ;
        when PGI_FOUR =>
                    alu_input_small_number <= "0100" ;
        when PGI_QUICK_IMMEDIATE =>
                    -- This is meaningful only for ADDQ instructions.
                    if ( instruction_register ( 11 downto 9 ) = "000" )
                    then
                        alu_input_small_number <= "1000" ;
                    else
                        alu_input_small_number ( 3 ) <= '0' ;
                        alu_input_small_number ( 2 downto 0 ) <=
                                instruction_register ( 11 downto 9 ) ;
                    end if ;
        when PGI_POSTINC_PREDEC =>
                    -- Calculate modifier for postinc/predec 
                    -- effective addresses.
                    case operation_size is
                    when BYTE =>
                            -- Byte operations on register 7 are treated
                            -- as word operations so as to preserve
                            -- stack alignment.
                            if ( ea_reg = "111" )
                            then
                                alu_input_small_number <= "0010" ;
                            else
                                alu_input_small_number <= "0001" ;
                            end if ;
                    when WORD =>
                            alu_input_small_number <= "0010" ;
                    when others => -- DWORD
                            alu_input_small_number <= "0100" ;
                    end case ;
        end case ;
    end process alu_input_muxes_2 ;

    alu_control_mux : process ( alu_mode , 
            alu_reverse_operands , instruction_register ) is
        variable sub_op : alu_internal_op_type ;
        variable cmp_op : alu_internal_op_type ;
        variable op     : alu_internal_op_type ;
    begin
        if ( alu_reverse_operands = '1' )
        then
            sub_op := ALU_INT_REV_SUB ;
            cmp_op := ALU_INT_REV_CMP ;
        else
            sub_op := ALU_INT_SUB ;
            cmp_op := ALU_INT_CMP ;
        end if ;

        -- The aim of this process is to translate an arithmetic
        -- operation type described in the opcode into something that
        -- the ALU can understand. There is a slightly different 
        -- translation for each type of instruction.
        case alu_mode is
        when ALU_I_FAMILY =>
                    case instruction_register ( 11 downto 9 ) is
                    when "000" =>
                            op := ALU_INT_OR ;
                    when "001" =>
                            op := ALU_INT_AND ;
                    when "010" =>
                            op := sub_op ;
                    when "011" =>
                            op := ALU_INT_ADD ;
                    when "101" =>
                            op := ALU_INT_EOR ;
                    when "110" =>
                            op := cmp_op ;
                    when others => -- op is (really) undefined.
                            op := ALU_INT_ADD ;
                    end case ;
                    alu_modify_ccrs <= '1' ;
        when ALU_Q_FAMILY|ALU_CLR_FAMILY =>
                    case instruction_register ( 8 ) is
                    when '0' =>
                            op := ALU_INT_ADD ;
                    when others =>
                            op := sub_op ;
                    end case ;
                    alu_modify_ccrs <= '1' ;
        when ALU_NO_FAMILY|ALU_A_FAMILY|ALU_X_FAMILY =>
                    case instruction_register ( 14 downto 12 ) is
                    when "000" =>
                            op := ALU_INT_OR ;
                    when "001" =>
                            op := sub_op ;
                    when "011" =>
                            if ( instruction_register ( 8 ) = '1' )
                            and ( alu_mode = ALU_NO_FAMILY )
                            then
                                op := ALU_INT_EOR ;
                            else
                                op := cmp_op ;
                            end if ;
                    when "100" =>
                            op := ALU_INT_AND ;
                    when "101" =>
                            op := ALU_INT_ADD ;
                    when others => -- op is (really) undefined.
                            op := ALU_INT_ADD ;
                    end case ;
                    alu_modify_ccrs <= '1' ;
        when ALU_ADD_UPDATE_CCRS =>
                    op := ALU_INT_ADD ;
                    alu_modify_ccrs <= '1' ;
        when ALU_ADD =>
                    op := ALU_INT_ADD ;
                    alu_modify_ccrs <= '0' ;
        when ALU_SUBTRACT =>
                    op := sub_op ;
                    alu_modify_ccrs <= '0' ;
        when others => -- op is (really) undefined.
                    op := ALU_INT_ADD ;
                    alu_modify_ccrs <= '0' ;
        end case ;
        alu_internal_op <= apply_alu_internal_op ( op ) ;
    
    end process alu_control_mux ;

