-- state_machine_controller.vhd
--
--


call_stack_pointer_plus_one <= call_stack_pointer + 1 ;
call_stack_pointer_minus_one <= call_stack_pointer - 1 ;

state_plus_one <= state + 1 ;



call_stack_ram : entity dp_ram
    generic map ( address_width => stack_pointer_register'length ,
            data_width => state_register'length )
    port map ( clock => clock , -- done in +ve cycle.
            write => write_enable ,
            address1 => call_stack_pointer_minus_one ,
            address2 => call_stack_pointer_minus_one ,
            data_in => value_to_be_stacked ,
            data_out1 => call_stack_at_ptr_minus_one ) ;


state_machine_controller : process (
        call_requested , reset , return_requested , clock ,
        state_plus_one , call_stack_at_ptr_minus_one ,
        call_stack_pointer_minus_one ,
        call_stack_pointer_plus_one ,
        call_state ) is
    constant zero_call_state : state_register := ( others => '0' ) ;
begin
    if ( clock'event )
    and ( clock = '0' )
    then
        if ( reset = '1' )
        then
            state <= ( others => '0' ) ;
            value_to_be_stacked <= ( others => '0' ) ;
            write_enable <= '0' ;
            call_stack_pointer <= ( others => '0' ) ;
        else
            -- CALL stacking operations
            value_to_be_stacked <= state_plus_one ;

            if ( call_requested = '0' )
            and ( return_requested = '0' )
            then
                -- CLOCK - just go to next state.
                state <= state_plus_one ;
                write_enable <= '0' ;
            elsif ( call_requested = '0' )
            and ( return_requested = '1' )
            then
                -- Return
                state <= call_stack_at_ptr_minus_one ;
                call_stack_pointer <= call_stack_pointer_minus_one ;
                write_enable <= '0' ;
            elsif ( call_requested = '1' )
            and ( return_requested = '0' )
            then
                -- a CALL
                if ( call_state = zero_call_state )
                then
                    -- This is a CALL NOTHING statement. 
                    -- This is a dummy call that means "just go to
                    -- the next state".
                    state <= state_plus_one ;
                    write_enable <= '0' ;
                else
                    -- This is a CALL to a real state machine.
                    state <= call_state ;
                    call_stack_pointer <= call_stack_pointer_plus_one ;
                    write_enable <= '1' ;
                end if ;
            else
                -- a JUMP. This is a call without a stack.
                -- In the positive going clock cycle, we change
                -- the state.
                state <= call_state ;
                write_enable <= '0' ;
            end if ;
        end if ;
    end if ;
end process state_machine_controller ;


