#include <stdio.h>
#include <string.h>

#include "exceptions.h"
#include "utils.h"
#include "ndfa_accept_state.h"


Basic_Exception :: Basic_Exception ( const char * msg )
{
    this -> msg = Copy_String ( msg ) ;
}

Basic_Exception :: ~Basic_Exception ()
{
    delete [] msg ;
}

SharedAcceptStateException :: SharedAcceptStateException 
                ( Accept_State * as1 , Accept_State * as2 ) :
                                Basic_Exception ()
{
    this -> as1 = as1 ;
    this -> as2 = as2 ; 
}

void UnavailableOpcodeException :: PrintMessage ()
{
    printf ( "The requested opcode is not available.\n" ) ;
}

void SharedAcceptStateException :: PrintMessage ()
{
    printf ( "The same bit pattern decodes to two different accept states!\n"
            "1st family:\n" ) ;
    as1 -> Print_Info ( stdout , "" ) ;
    printf ( "2nd family:\n" ) ;
    as2 -> Print_Info ( stdout , "" ) ;
}


void File_Access_Exception :: PrintMessage ()
{
    printf ( "Unable to access file '%s'.\n" , msg ) ;
}

void Dir_Access_Exception :: PrintMessage ()
{
    printf ( "Unable to list directory '%s'.\n" , msg ) ;
}

void Duplicate_Label_Exception :: PrintMessage ()
{
    printf ( "The label '%s' is duplicated.\n" , msg ) ;
}

void Unavailable_Microsub_Exception :: PrintMessage ()
{
    printf ( "The label '%s' cannot be found.\n" , msg ) ;
}

void Unrecognised_Opcode_Map_Entry :: PrintMessage ()
{
    printf ( "Unrecognised opcode map entry:\n%s\n" , msg ) ;
}

void No_Opcodes_Read_Exception :: PrintMessage ()
{
    printf ( "No required opcodes could be read from %s.\n"
            "Ensure that the file is in a supported format.\n" , msg ) ;
}

void Unrecognised_Optimisation_Exception :: PrintMessage ()
{
    printf ( "Unrecognised optimisation type: %s\n" , msg ) ;
}

void Missing_Clock_Exception :: PrintMessage ()
{
    printf ( "State machine %s has some instructions in\n"
            "the final state, but no CLOCK statement.\n" , msg ) ;
}

void Empty_Machine_Exception :: PrintMessage ()
{
    printf ( "State machine %s appears to be empty.\n" , msg ) ;
}


void Unrecognised_Ini_File_Entry :: PrintMessage ()
{
    printf ( "The following ini file entry is unrecognised. Check that it is\n"
        "a valid variable assignment: %s\n" , msg ) ;
}

void No_Setting_For_Variable :: PrintMessage ()
{
    printf ( "There is no setting for required variable '%s' in the\n"
        "provided ini file.\n" , msg ) ;
}



