
#include "opcode_database.h"
#include "state_machine_loader.h"
#include "exceptions.h"
#include "control.h"
#include "utils.h"
#include "settings.h"

int main ( int argc , char * argv [] )
{

    try {
        Settings * s ;

        g_verbose_setting = VERBOSE_MEDIUM ;

        MESSAGE ( "  ** State Machine Compiler **\n"
                "Binary build time: " __TIME__ " " __DATE__ "\n\n" ) ;

        /* Read the settings */
        if ( argc > 1 )
        {
            s = new Settings ( argv [ 1 ] ) ;
            if ( argc > 2 )
            {
                s -> Set_Required_Opcode_File ( argv [ 2 ] ) ;
            }
        } else {
            s = new Settings ( "smc.ini" ) ;
        }

        g_verbose_setting = s -> Get_Verbose_Level () ;

        Control c ( s -> Get_Opcode_Map_Filename () ,
                    s -> Get_State_Machine_Directory () ,
                    s -> Get_Root_VHDL_Input_Filename () ,
                    s -> Get_VHDL_Input_Directory () ,
                    s -> Get_Initial_State_Machine () ,
                    s -> Get_Required_Opcode_File () ) ;
        
        FILE * fd = fopen ( s -> Get_Output_Filename () , "wt" ) ;
        if ( fd == 0L )
        {
            throw new File_Access_Exception ( s -> Get_Output_Filename () ) ;
        }
        c . Generate_VHDL ( fd ) ;
        fclose ( fd ) ;
        
        MESSAGE ( "Completed successfully.\n" ) ;

        delete s ;

        return 0 ;
    } catch ( Basic_Exception * e )
    {
        MESSAGE ( "An exception was thrown:\n" ) ;
        e -> PrintMessage () ;

        return 1 ;
    }
}

